"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComplexObjectBuilder = void 0;
const react_1 = require("@fluentui/react");
const PropertyType_1 = require("../../models/property/PropertyType");
const FormBuilder_1 = require("../../components/FormBuilder");
const validationMessage_1 = require("../../components/validationMessage");
const DynamicArrayField_1 = require("./components/DynamicArrayField");
const react_2 = __importDefault(require("react"));
const ValidationUtil_1 = require("../../utils/common/ValidationUtil");
class ComplexObjectBuilder {
    constructor(labelRender) {
        this.id = "internal_custombuilder";
        this._labelRender = (propertySchema, key) => react_2.default.createElement(react_1.Label, { key: key }, propertySchema.displayName);
        // property overrides cannot be nested for now (ewi)
        this.convertPropertyOverrides = (propertyOverrides, property) => {
            if (propertyOverrides == undefined)
                return undefined;
            let clone = Object.assign({}, propertyOverrides);
            if (Array.isArray(propertyOverrides.disabledProps) && propertyOverrides.disabledProps.indexOf(property) >= 0)
                clone.disabledProps = true;
            return clone;
        };
        this.build = (renderProps, property, schema) => {
            var _a;
            let { item, onChange, onBlur, validationResults, validationResultPrefix } = renderProps;
            if (item == null)
                throw Error("item is null");
            if (schema == null)
                throw Error("schema is null");
            let validationIndexer = validationResultPrefix ? `${validationResultPrefix}.${property}` : property;
            let key = `${validationResultPrefix !== null && validationResultPrefix !== void 0 ? validationResultPrefix : ""}${property}`;
            let props = {
                key: (_a = schema.key) !== null && _a !== void 0 ? _a : key,
                value: item[property],
                options: schema,
                onChange: (value) => onChange(property, value),
                onBlur: (value) => onBlur(property, item[property]),
                disabled: schema.disable ? schema.disable(item) : false,
                errorMessage: ValidationUtil_1.validationUtil.getResultAsString(validationResults[validationIndexer]),
                parent: item
            };
            const WrapInLabel = (element, addErrormessage) => {
                return (react_2.default.createElement("div", { className: "formbuilder-property", key: props.key },
                    !props.options.hideLabel && this._labelRender(schema, key + "-label"),
                    element,
                    (addErrormessage && props.errorMessage) && (0, validationMessage_1.validationMessageElement)(props.errorMessage)));
            };
            switch (schema.propertyType) {
                case PropertyType_1.PropertyType.Array: return { found: true, element: WrapInLabel(react_2.default.createElement(DynamicArrayField_1.DynamicArrayField, Object.assign({}, schema, props))) };
                case PropertyType_1.PropertyType.Custom:
                    let customConfig = schema.config;
                    if (!(customConfig === null || customConfig === void 0 ? void 0 : customConfig.componentName))
                        return { found: false, element: undefined };
                    // const Comp = this.customComponents[customConfig?.componentName];
                    const Comp = customConfig.component;
                    return { found: true, element: react_2.default.createElement(react_2.default.Fragment, null, WrapInLabel(react_2.default.createElement(Comp, Object.assign({ key: key }, props, schema)))) };
                case PropertyType_1.PropertyType.FormItem:
                    let formConfig = schema.config;
                    if (!(formConfig === null || formConfig === void 0 ? void 0 : formConfig.itemType))
                        return { found: false, element: undefined };
                    let formItemProps = {
                        item: item[property],
                        itemType: formConfig.itemType,
                        groupContainer: formConfig.groupContainer,
                        groupRender: formConfig.groupRender,
                        onPropertyChange: (item, prop, value) => props.onChange(item),
                        validationResult: validationResults,
                        validationResultPrefix: property,
                        propertyOverrides: this.convertPropertyOverrides(renderProps.propertyOverrides, property)
                    };
                    let addErrormessage = false;
                    // used for having properties as array of formitem
                    let listProps = schema === null || schema === void 0 ? void 0 : schema.listProps;
                    if (listProps != undefined) {
                        formItemProps.onPropertyChange = undefined;
                        listProps.config.onItemsChange = items => props.onChange(items);
                        addErrormessage = true;
                    }
                    return { found: true, element: react_2.default.createElement(react_2.default.Fragment, null, WrapInLabel(react_2.default.createElement(FormBuilder_1.FormBuilder, Object.assign({ key: key, keyPrefix: key }, formItemProps, formConfig, { listProps: schema === null || schema === void 0 ? void 0 : schema.listProps })), addErrormessage)) };
                default: return { found: false, element: undefined };
            }
        };
        if (labelRender)
            this._labelRender = labelRender;
    }
}
exports.ComplexObjectBuilder = ComplexObjectBuilder;
ComplexObjectBuilder.Create = (labelRender) => {
    let builder = new ComplexObjectBuilder(labelRender);
    return builder;
};
//# sourceMappingURL=ComplexObjectBuilder.js.map