"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("@fluentui/react");
const react_2 = require("react");
const useStateRef_1 = require("../../../../hooks/useStateRef");
const IFormBuilderListConfig_1 = require("../../../../components/config/IFormBuilderListConfig");
const Searcher_1 = require("./components/search/Searcher");
const Language_1 = require("../../../../models/language/Language");
const ValidationResult_1 = require("../../../../models/validation/ValidationResult");
const FluentDialog_1 = require("./FluentDialog");
const react_3 = __importDefault(require("react"));
const FluentList = (props) => {
    var _a, _b, _c, _d;
    const editorFormRef = (0, react_2.useRef)();
    const [items, setItems, itemsRef] = (0, useStateRef_1.useStateRef)(props.items);
    const [filteredItems, setFilteredItems, filteredItemsRef] = (0, useStateRef_1.useStateRef)([...props.items]);
    const [showEditor, setShowEditor] = (0, react_2.useState)(false);
    const [selectedItems, setSelectedItems, selectedItemsRef] = (0, useStateRef_1.useStateRef)([]);
    const [columns, setColumns, columnRef] = (0, useStateRef_1.useStateRef)();
    const [showValidationOverrideConfirm, setShowValidationOverrideConfirm] = (0, react_2.useState)(false);
    const [enableShimmer, setEnableShimmer] = (0, react_2.useState)(false);
    const [newItemMode, setNewItemMode] = (0, react_2.useState)(false);
    const [menuItems, setMenuItems] = (0, react_2.useState)(undefined);
    const [selection] = (0, react_2.useState)(new react_1.Selection({
        selectionMode: props.listProps.config.multiSelect ? react_1.SelectionMode.multiple : react_1.SelectionMode.single,
        getKey: (item, index) => props.listProps.config.itemIdentifier(item),
        onSelectionChanged: () => {
            setSelectedItems(selection.getSelection());
        }
    }));
    (0, react_2.useEffect)(() => {
        let cols = buildColumns();
        setColumns(cols);
    }, [props.schema]);
    (0, react_2.useEffect)(() => {
        setItems(props.items);
    }, [props.items]);
    (0, react_2.useEffect)(() => {
        var _a, _b;
        if (!!((_a = props.listProps.config) === null || _a === void 0 ? void 0 : _a.shimmerLines) && !!!((_b = props.items) === null || _b === void 0 ? void 0 : _b.length)) {
            setEnableShimmer(true);
            setTimeout(() => {
                setEnableShimmer(false);
            }, 5000);
        }
        else
            setEnableShimmer(false);
    }, [props.items, (_a = props.listProps.config) === null || _a === void 0 ? void 0 : _a.shimmerLines]);
    // TODO: move out to FormList (ewi)
    const isMenuItemDisabled = (selectionMode) => {
        let disabled = true;
        if ((selectionMode & IFormBuilderListConfig_1.IFormBuilderListMenuItemSelectionMode.None) === IFormBuilderListConfig_1.IFormBuilderListMenuItemSelectionMode.None)
            disabled = selectedItems.length !== 0;
        if ((selectionMode & IFormBuilderListConfig_1.IFormBuilderListMenuItemSelectionMode.Single) === IFormBuilderListConfig_1.IFormBuilderListMenuItemSelectionMode.Single)
            disabled = disabled && selectedItems.length !== 1;
        if ((selectionMode & IFormBuilderListConfig_1.IFormBuilderListMenuItemSelectionMode.Multi) === IFormBuilderListConfig_1.IFormBuilderListMenuItemSelectionMode.Multi)
            disabled = disabled && selectedItems.length < 2;
        return disabled;
    };
    (0, react_2.useEffect)(() => {
        var _a, _b;
        if (((_a = props.listProps.menuConfig) === null || _a === void 0 ? void 0 : _a.actions) == null)
            return;
        let actions = (_b = props.listProps.menuConfig) === null || _b === void 0 ? void 0 : _b.actions(createItem, editItem, deleteItems);
        if (!(actions === null || actions === void 0 ? void 0 : actions.length))
            return; // is it needed to set to empty array as well? (ewi)
        // parse to commandbarProps
        let commandBarItems = actions.map(_ => {
            return {
                key: _.title,
                text: _.title,
                onClick: () => _.action(selectedItems),
                disabled: isMenuItemDisabled(_.selectionMode),
                iconProps: { iconName: _.iconName }
            };
        });
        setMenuItems(commandBarItems);
    }, [props.listProps.menuConfig, selectedItems]);
    // TODO: make this usable for delete as well (ewi)
    // TODO: move out to FormList (ewi)
    const itemChangeInCollection = (collection, item) => {
        let clone = [...collection];
        let itemIndex = clone.findIndex(_ => props.listProps.config.itemIdentifier(_) == props.listProps.config.itemIdentifier(item));
        if (itemIndex < 0)
            clone.unshift(item);
        else
            clone[itemIndex] = Object.assign({}, item);
        return clone;
    };
    // TODO: move out to FormList (ewi)
    const onItemChange = (item) => {
        if (item == null)
            return;
        // items
        let changedItems = itemChangeInCollection(itemsRef.current, item);
        setItems(changedItems);
        // // filtered
        let changedFilteredItems = itemChangeInCollection(filteredItemsRef.current, item);
        setFilteredItems(changedFilteredItems);
        if (props.listProps.onItemChange)
            props.listProps.onItemChange(Object.assign({}, item));
        if (props.listProps.config.onItemsChange)
            props.listProps.config.onItemsChange([...changedItems]);
    };
    // TODO: move out to FormList (ewi)
    const filterColumns = (propKeys) => {
        var _a;
        let options = (_a = props.listProps.columnConfig) === null || _a === void 0 ? void 0 : _a.columnsPicks;
        if ((options === null || options === void 0 ? void 0 : options.columns) == null)
            return propKeys;
        if (options.pickType == IFormBuilderListConfig_1.IFormListColumnsPickType.Only)
            return propKeys.filter(_ => options === null || options === void 0 ? void 0 : options.columns.includes(_));
        else if (options.pickType == IFormBuilderListConfig_1.IFormListColumnsPickType.Without)
            return propKeys.filter(_ => !(options === null || options === void 0 ? void 0 : options.columns.includes(_)));
        else
            return propKeys;
    };
    // TODO: move out to FormList (ewi)
    const customValueRender = (propInfo) => {
        var _a;
        let valueRender = (_a = propInfo === null || propInfo === void 0 ? void 0 : propInfo.listItemOptions) === null || _a === void 0 ? void 0 : _a.customValueRender;
        if (valueRender == undefined)
            return undefined;
        else {
            return (item) => valueRender(item, ((item) => onItemChange(item)));
        }
    };
    // TODO: move out to FormList (ewi)
    //       return type and map func
    const buildColumns = () => {
        var _a;
        let propertyOptions = props.schema.options.properties;
        let propKeys = Object.keys(propertyOptions);
        let filteredPropKeys = filterColumns(propKeys);
        let columns = filteredPropKeys.map(_ => {
            var _a, _b;
            let propInfo = propertyOptions[_];
            let col = {
                key: _,
                fieldName: _,
                name: propInfo.displayName,
                onRender: customValueRender(propInfo),
                minWidth: (_a = propInfo.listItemOptions) === null || _a === void 0 ? void 0 : _a.minWidth,
                isResizable: (_b = propInfo.listItemOptions) === null || _b === void 0 ? void 0 : _b.isResizable,
                onColumnClick: (ev, col) => {
                    onColumnClick(ev, col);
                }
            };
            return col;
        });
        let sortOrder = (_a = props.listProps.columnConfig) === null || _a === void 0 ? void 0 : _a.columnOrder;
        if (sortOrder != null)
            return columns.sort((a, b) => {
                if (sortOrder.indexOf(a.fieldName) == -1)
                    return 1;
                if (sortOrder.indexOf(b.fieldName) == -1)
                    return -1;
                return sortOrder.indexOf(a.fieldName) - sortOrder.indexOf(b.fieldName);
            });
        else
            return columns;
    };
    // TODO: move out to FormList (ewi)
    //       name: sortByColumn
    const onColumnClick = (ev, column) => {
        if (columnRef.current == null)
            return;
        const newColumns = columnRef.current.slice();
        const currColumn = columnRef.current.filter(currCol => column.key === currCol.key)[0];
        newColumns.forEach((newCol) => {
            if (newCol === currColumn) {
                currColumn.isSortedDescending = !currColumn.isSortedDescending;
                currColumn.isSorted = true;
            }
            else {
                newCol.isSorted = false;
                newCol.isSortedDescending = true;
            }
        });
        const newItems = copyAndSort(filteredItemsRef.current, currColumn.fieldName, currColumn.isSortedDescending);
        setColumns(newColumns);
        setFilteredItems(newItems);
    };
    // TODO: move out to FormList (ewi)
    const copyAndSort = (items, columnKey, isSortedDescending) => {
        let key = columnKey;
        key = key.replace("column-", ""); // replace prefix from column key
        return items.slice(0).sort((a, b) => ((isSortedDescending ? a[key] < b[key] : a[key] > b[key]) ? 1 : -1));
    };
    // TODO: move crud operations out to FormList (ewi)
    //       with alot of other stuff as well
    const createItem = (pre) => {
        let item = {};
        if (pre) {
            let result = pre(item);
            if (result === false)
                return;
        }
        selection.setAllSelected(false);
        setSelectedItems([item]);
        setNewItemMode(true);
        setShowEditor(true);
    };
    const editItem = (pre) => {
        let sItem = Object.assign({}, selectedItemsRef.current[0]);
        if (pre) {
            let result = pre(sItem);
            if (result === false)
                return;
        }
        setSelectedItems([sItem]);
        setTimeout(() => {
            setNewItemMode(false);
            setShowEditor(true);
        }, 400);
    };
    // TODO: show confirmation dialog (ewi)
    // TODO: move out to FormList (ewi)
    const deleteItems = (pre) => {
        let sItems = [...selectedItems];
        // pre event (can cancel the operation)
        if (pre) {
            let result = pre(sItems);
            if (result === false)
                return;
        }
        let itemsLeft = items.filter(_ => sItems.findIndex(si => props.listProps.config.itemIdentifier(_) == props.listProps.config.itemIdentifier(si)) < 0);
        setItems(itemsLeft);
        let filteredItemsLeft = filteredItems.filter(_ => sItems.findIndex(si => props.listProps.config.itemIdentifier(_) == props.listProps.config.itemIdentifier(si)) < 0);
        setFilteredItems(filteredItemsLeft);
        setSelectedItems([]);
        selection.setAllSelected(false);
        if (props.listProps.config.onItemsChange)
            props.listProps.config.onItemsChange([...itemsLeft]);
    };
    const validateForm = () => __awaiter(void 0, void 0, void 0, function* () {
        var _e;
        let validated = yield ((_e = editorFormRef.current) === null || _e === void 0 ? void 0 : _e.validateItem());
        return validated;
    });
    const saveForm = () => __awaiter(void 0, void 0, void 0, function* () {
        var _f;
        let validated = yield validateForm();
        if (validated == ValidationResult_1.ValidationResult.FailedDontBlock) {
            setShowValidationOverrideConfirm(true);
        }
        else if (validated == ValidationResult_1.ValidationResult.Success) {
            onItemChange((_f = editorFormRef.current) === null || _f === void 0 ? void 0 : _f.getItem());
            setShowEditor(false);
        }
    });
    const dismissForm = () => __awaiter(void 0, void 0, void 0, function* () {
        if (newItemMode)
            setSelectedItems([]);
        setNewItemMode(false);
        setShowEditor(false);
    });
    const getItem = () => {
        var _a;
        return (_a = editorFormRef.current) === null || _a === void 0 ? void 0 : _a.getItem();
    };
    const onRenderFooterContent = (0, react_2.useCallback)(() => {
        var _a, _b;
        if (((_b = (_a = props.listProps) === null || _a === void 0 ? void 0 : _a.editorConfig) === null || _b === void 0 ? void 0 : _b.customFooter) != undefined)
            return props.listProps.editorConfig.customFooter(saveForm, dismissForm, validateForm, getItem);
        else
            return (react_3.default.createElement("div", null,
                react_3.default.createElement(react_1.PrimaryButton, { text: newItemMode ? Language_1.lang.texts.areas.common.create : Language_1.lang.texts.areas.common.save, styles: { root: { marginRight: 8 } }, onClick: saveForm }),
                react_3.default.createElement(react_1.DefaultButton, { text: Language_1.lang.texts.areas.common.cancel, onClick: dismissForm })));
    }, [showEditor]);
    const handleColumnReorder = (draggedIndex, targetIndex) => {
        if (columns == null)
            return;
        const draggedItems = columns[draggedIndex];
        const newColumns = [...columns];
        // insert before the dropped item
        newColumns.splice(draggedIndex, 1);
        newColumns.splice(targetIndex, 0, draggedItems);
        setColumns(newColumns);
    };
    // TODO: refactor the shit out of everything in this component
    //       take it completely apart, this has been an 'is it even possible' implementation (ewi)
    // TODO: move Panel and Dialog out to files (ewi)
    // TODO: move Form out to file and reuse in Panel and Dialog (ewi)
    return react_3.default.createElement(react_3.default.Fragment, null,
        (showEditor && (selectedItems === null || selectedItems === void 0 ? void 0 : selectedItems.length))
            && react_3.default.createElement(FluentDialog_1.FluentDialog, { item: selectedItems[0], editorConfig: props.listProps.editorConfig, formListProps: props, keyPrefix: (_b = props.keyPrefix) !== null && _b !== void 0 ? _b : "", newItemMode: newItemMode, onRenderFooterContent: onRenderFooterContent, show: (show) => setShowEditor(show), dialogType: (_c = props.listProps.editorConfig) === null || _c === void 0 ? void 0 : _c.type, ref: editorFormRef }),
        react_3.default.createElement("div", { className: "formbuilder-listcontainer-toolbar" },
            ((_d = props.listProps.searchConfig) === null || _d === void 0 ? void 0 : _d.searchEnabled) &&
                react_3.default.createElement(Searcher_1.Searcher, { items: items, onSearch: (filteredItems, searchText) => {
                        setFilteredItems(filteredItems);
                        selection.setAllSelected(false);
                        setSelectedItems([]);
                    }, className: "tp-searcher", delay: 600, searchPlaceHolder: props.listProps.searchConfig.searchPlaceHolder, enableTextHighlight: true, searchFields: props.listProps.searchConfig.searchableFields, highLightElements: () => Array.from(document.querySelectorAll('.ms-List .ms-DetailsRow-cell:not(.ms-DetailsRow-cellCheck)'))
                        .filter(_ => {
                        var _a;
                        if (((_a = props.listProps.searchConfig) === null || _a === void 0 ? void 0 : _a.searchableFields) == null)
                            return [];
                        return props.listProps.searchConfig.searchableFields.findIndex(sf => sf == _.attributes["data-automation-key"].value) >= 0;
                    }), filterOnItemsChange: true }),
            menuItems !== undefined &&
                react_3.default.createElement(react_1.CommandBar, { items: menuItems, styles: { root: { padding: 0 } } })),
        react_3.default.createElement("div", { className: "formbuilder-listcontainer-list" },
            react_3.default.createElement(react_1.ShimmeredDetailsList, { columns: columns, items: filteredItems, 
                // selectionPreservedOnEmptyClick
                selection: selection, enableShimmer: enableShimmer, shimmerLines: props.listProps.config.shimmerLines, layoutMode: react_1.DetailsListLayoutMode.fixedColumns, constrainMode: react_1.ConstrainMode.horizontalConstrained, setKey: 'itemId', 
                // setKey="none"
                getKey: (item) => item != null ? `item-${props.listProps.config.itemIdentifier(item)}` : "item-null", 
                // useReducedRowRenderer
                onItemInvoked: props.listProps.config.disableItemInvoke ? undefined :
                    (item, index, ev) => {
                        if (item == null) {
                            setSelectedItems([]);
                        }
                        else {
                            setSelectedItems([item]);
                        }
                        editItem();
                    }, columnReorderOptions: (columns === null || columns === void 0 ? void 0 : columns.length) ? { handleColumnReorder } : undefined })),
        (!enableShimmer && !!!(items === null || items === void 0 ? void 0 : items.length)) && react_3.default.createElement(react_1.Label, null, "No items found"),
        react_3.default.createElement(react_1.Dialog, { hidden: !showValidationOverrideConfirm, title: Language_1.lang.texts.areas.list.validationRequirementsNotMet.title, onDismiss: () => setShowValidationOverrideConfirm(false), dialogContentProps: {
                subText: Language_1.lang.texts.areas.list.validationRequirementsNotMet.desctiption
            }, modalProps: {
                isBlocking: true
            } },
            react_3.default.createElement(react_1.DialogFooter, null,
                react_3.default.createElement(react_1.PrimaryButton, { text: Language_1.lang.texts.areas.list.validationRequirementsNotMet.ok, onClick: () => {
                        var _a;
                        setShowValidationOverrideConfirm(false);
                        setShowEditor(false);
                        onItemChange((_a = editorFormRef.current) === null || _a === void 0 ? void 0 : _a.getItem());
                    } }),
                react_3.default.createElement(react_1.DefaultButton, { text: Language_1.lang.texts.areas.list.validationRequirementsNotMet.cancel, onClick: () => setShowValidationOverrideConfirm(false) }))));
};
exports.default = FluentList;
//# sourceMappingURL=FluentList.js.map