import React from 'react';
import { IFormItemOptions } from '../models/options/IFormItemOptions';
import { IFormSchema } from '../models/schema/IFormSchema';
import { IValidationResult } from '../models/validation/IValidationResult';
import { ValidationEventType } from '../models/validation/ValidationEventType';
import { ValidationOverride } from '../models/validation/ValidationOverride';
import { ValidationResult } from '../models/validation/ValidationResult';
import { IFormItem } from '../modules/IFormItem';
import { IDynamicSchemaConfig } from './config/IFormBuilderListConfig';
import { IFormGrouping } from '../interfaces/IFormGrouping';
import { IPropertyOverrides } from '../interfaces/IPropertyOverrides';
import { ILoadingSpinnerProps } from '../builders/fluentUI/components/list/components/FluentFormShimmer';
export interface IFormItemProps<T extends IFormItem> {
    itemType: new () => T;
    item: T;
    schema?: IFormSchema<T>;
    dynamicSchema?: IDynamicSchemaConfig<T>;
    onPropertyChange?: (item: IFormItem, prop: string, value: any) => void;
    groupContainer?: React.ElementType<{
        groupings: Array<IFormGrouping>;
    }>;
    groupRender?: (grouping: IFormGrouping, children: Array<any>) => JSX.Element;
    validationOverride?: ValidationOverride;
    formItemConfigOverrides?: Partial<IFormItemOptions<T>>;
    propertyOverrides?: IPropertyOverrides;
    spinnerProps?: ILoadingSpinnerProps;
    /*** DO NOT USE! only used internally */
    validationResult?: IValidationResult;
    /*** DO NOT USE! only used internally */
    validationResultPrefix?: string;
    keyPrefix?: string;
}
export declare type FormRef<T extends IFormItem> = {
    getItem: () => T;
    validateItem: (type?: ValidationEventType) => Promise<ValidationResult>;
};
export declare const Form: React.ForwardRefExoticComponent<IFormItemProps<IFormItem> & React.RefAttributes<unknown>>;
