"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Form = void 0;
const react_1 = __importStar(require("react"));
const useIsMounted_1 = __importDefault(require("../hooks/useIsMounted"));
const ValidationEventType_1 = require("../models/validation/ValidationEventType");
const ValidationOverride_1 = require("../models/validation/ValidationOverride");
const ValidationResult_1 = require("../models/validation/ValidationResult");
const FormValidator_1 = require("./helpers/FormValidator");
const FormBuilderInitializer_1 = require("../builders/helpers/FormBuilderInitializer");
const ValidationUtil_1 = require("../utils/common/ValidationUtil");
const SchemaFetch_1 = require("../utils/common/SchemaFetch");
const EmptyGroupContainer = props => {
    return react_1.default.createElement("div", { key: `emptygroup-${props.itemId}` }, props.children);
};
exports.Form = (0, react_1.forwardRef)((props, ref) => {
    var _a, _b, _c;
    if (!FormBuilderInitializer_1.formbuilder.verify())
        throw Error("Formbuilder not initialized! call 'formBuilder.initialize' as early as possible");
    const isMounted = (0, useIsMounted_1.default)();
    const [item, setItem] = (0, react_1.useState)(props.item);
    const [schema, setSchema] = (0, react_1.useState)(props.dynamicSchema != null ? undefined : props.schema);
    const dynamicSchema = (0, react_1.useRef)();
    const [groupings, setGroupings] = (0, react_1.useState)();
    const [groupsHandled, setGroupsHandled] = (0, react_1.useState)(false);
    const [validationResults, setValidationResults] = (0, react_1.useState)((_a = props.validationResult) !== null && _a !== void 0 ? _a : {});
    const validateItem = (formItem, schema, on = ValidationEventType_1.ValidationEventType.Manual, prop) => __awaiter(void 0, void 0, void 0, function* () {
        var _d;
        let schemaTest = (_d = dynamicSchema.current) !== null && _d !== void 0 ? _d : schema;
        switch (props.validationOverride) {
            case undefined:
            case ValidationOverride_1.ValidationOverride.None:
                let results = yield FormValidator_1.formValidator.validate(formItem, prop, schemaTest, on, validationResults, props.validationResultPrefix);
                setValidationResults(Object.assign({}, results));
                return ValidationUtil_1.validationUtil.validated(results) ? ValidationResult_1.ValidationResult.Success : ValidationResult_1.ValidationResult.Failed;
            case ValidationOverride_1.ValidationOverride.Ignore:
                setValidationResults({});
                return ValidationResult_1.ValidationResult.Success;
            case ValidationOverride_1.ValidationOverride.Continue:
                let results2 = yield FormValidator_1.formValidator.validate(formItem, prop, schemaTest, on, validationResults, props.validationResultPrefix);
                setValidationResults(Object.assign({}, results2));
                return ValidationUtil_1.validationUtil.validated(results2) ? ValidationResult_1.ValidationResult.Success : ValidationResult_1.ValidationResult.FailedDontBlock;
            default:
                return ValidationResult_1.ValidationResult.Success;
        }
    });
    (0, react_1.useImperativeHandle)(ref, () => ({
        getItem: () => item,
        validateItem: (type) => validateItem(item, props.schema, type, undefined)
    }), [item, props.schema]);
    const getDynamicSchemaKey = (it) => {
        var _a;
        let dynamicKey = (_a = props.dynamicSchema) === null || _a === void 0 ? void 0 : _a.dynamicKey;
        if (dynamicKey == null)
            return undefined;
        return dynamicKey(it !== null && it !== void 0 ? it : item);
    };
    const loadSchema = (it, force) => __awaiter(void 0, void 0, void 0, function* () {
        var _e, _f, _g, _h;
        if (schema !== undefined && ((_e = props.dynamicSchema) === null || _e === void 0 ? void 0 : _e.dynamicKey) == null && !force)
            return;
        let schemaMerged = yield (0, SchemaFetch_1.fetchSchema)((_g = (_f = props.dynamicSchema) === null || _f === void 0 ? void 0 : _f.customFormType) !== null && _g !== void 0 ? _g : props.itemType, props.formItemConfigOverrides, props.propertyOverrides, getDynamicSchemaKey(it));
        if (((_h = props.dynamicSchema) === null || _h === void 0 ? void 0 : _h.customFormType) != null) {
            dynamicSchema.current = schemaMerged;
        }
        if (!isMounted())
            return;
        setSchema(schemaMerged);
    });
    (0, react_1.useEffect)(() => {
        loadSchema();
    }, [props.itemType]);
    (0, react_1.useEffect)(() => {
        if (props.propertyOverrides == null)
            return;
        loadSchema(undefined, true);
    }, [props.propertyOverrides]);
    const GroupContainer = (_b = props.groupContainer) !== null && _b !== void 0 ? _b : EmptyGroupContainer;
    (0, react_1.useEffect)(() => {
        buildGroupings();
    }, [schema]);
    const buildGroupings = () => {
        var _a, _b;
        if (((_a = schema === null || schema === void 0 ? void 0 : schema.options) === null || _a === void 0 ? void 0 : _a.groups) == null) {
            setGroupsHandled(true);
            return;
        }
        let groups = (_b = schema === null || schema === void 0 ? void 0 : schema.options) === null || _b === void 0 ? void 0 : _b.groups;
        let groupedProps = {};
        Object.keys(schema.options.properties).forEach(_ => {
            var _a;
            let prop = schema.options.properties[_];
            let groupKey = (_a = prop.group) !== null && _a !== void 0 ? _a : "";
            if (groupedProps[groupKey] == null)
                groupedProps[groupKey] = [];
            groupedProps[groupKey].push(_);
        });
        let propertyGroupings = Object.keys(groupedProps).map(_ => {
            let group = Object.keys(groups).length && groups[_] ? groups[_] : { displayName: "Not grouped" }; // TODO: fuck the stupid typescript linter (ewi)
            return {
                groupKey: _,
                displayName: group.displayName,
                groupProps: Object.assign({}, group.groupProps),
                properties: groupedProps[_]
            };
        });
        setGroupings(propertyGroupings);
        setGroupsHandled(true);
    };
    const onPropertyChange = (property, value) => {
        var _a;
        let clone = Object.assign({}, item);
        clone[property] = value;
        setItem(clone);
        validateItem(clone, schema, ValidationEventType_1.ValidationEventType.Change, property);
        var onChanged = (_a = schema === null || schema === void 0 ? void 0 : schema.options.properties[property]) === null || _a === void 0 ? void 0 : _a.onChanged;
        if (onChanged != null) {
            let result = onChanged(clone);
            if (result === null || result === void 0 ? void 0 : result.refreshDynamicSchema) {
                loadSchema(clone);
            }
        }
        if (props.onPropertyChange)
            props.onPropertyChange(clone, property, value);
    };
    const Spinner = FormBuilderInitializer_1.formbuilder.formItemRender.spinner();
    if (item == null)
        return null;
    if (schema == null)
        return react_1.default.createElement(Spinner, Object.assign({}, props.spinnerProps));
    if (!groupsHandled)
        return null;
    const propertyRenderProps = {
        item: item,
        schema: schema.options,
        onChange: onPropertyChange,
        onBlur: (prop) => validateItem(item, schema, ValidationEventType_1.ValidationEventType.Blur, prop),
        validationResults: validationResults,
        validationResultPrefix: props.validationResultPrefix,
        // propertyOverrides: props.propertyOverrides // this is merged with the schema, and shouldnt be part of the child items (ewi)
    };
    return (react_1.default.createElement("div", { className: "formbuilder-formcontainer", key: `${props.keyPrefix}-formbuilder-formcontainer` },
        react_1.default.createElement(GroupContainer, { groupings: groupings, key: `${props.keyPrefix}-formbuilder-groupcontainer` }, (_c = groupings === null || groupings === void 0 ? void 0 : groupings.map(_ => {
            let children = FormBuilderInitializer_1.formbuilder.formItemRender.properties(propertyRenderProps, _.properties);
            if (children === undefined)
                return null;
            return props.groupRender ? props.groupRender(_, children) : children;
        })) !== null && _c !== void 0 ? _c : FormBuilderInitializer_1.formbuilder.formItemRender.properties(propertyRenderProps) // not defining properties here, defaults to all
        )));
});
//# sourceMappingURL=Form.js.map