"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildFormItemRender = void 0;
const buildFormItemRender = () => {
    var _builders = [];
    const hasBuilders = () => _builders.length > 0;
    const register = (builder) => {
        if (_builders.findIndex(_ => _.id === builder.id) >= 0)
            return;
        _builders.push(builder);
    };
    const registerRange = (builders) => {
        builders === null || builders === void 0 ? void 0 : builders.forEach(_ => register(_));
    };
    const list = () => {
        for (const builder of _builders) {
            if (builder.listComponent)
                return builder.listComponent();
        }
        return undefined;
    };
    const spinner = () => {
        for (const builder of _builders) {
            if (builder.loadingSpinnerComponent)
                return builder.loadingSpinnerComponent();
        }
        return undefined;
    };
    const properties = (renderProps, properties) => {
        let elements = [];
        // loop through properties specified or defautls to all properties
        for (const prop of properties !== null && properties !== void 0 ? properties : Object.keys(renderProps.schema.properties)) {
            let propSchema = Object.assign({}, renderProps.schema.properties[prop]);
            if (propSchema.hide && propSchema.hide(renderProps.item))
                continue;
            for (const builder of _builders) {
                let result = builder.build(renderProps, prop, propSchema);
                if (result.found && result.element != null)
                    elements.push(result.element);
            }
        }
        return (elements === null || elements === void 0 ? void 0 : elements.length) ? elements : undefined;
    };
    return { register, registerRange, properties, hasBuilders, list, spinner };
};
exports.buildFormItemRender = buildFormItemRender;
//# sourceMappingURL=FormItemRender.js.map