"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.formValidator = void 0;
const FormSchemaUtil_1 = require("../../utils/common/FormSchemaUtil");
const validateRule = (item, property, rule, onEvent, results, validationResultPrefix) => __awaiter(void 0, void 0, void 0, function* () {
    if (rule == null)
        return;
    let propertyIndexer = validationResultPrefix ? `${validationResultPrefix}.${property}` : property;
    if (Array.isArray(rule)) {
        rule.forEach(_ => {
            let validated = (onEvent != undefined && !((_.validateOn & onEvent) === onEvent)) || (!_.disabled && _.validationRule(item));
            if (results[propertyIndexer] == null)
                results[propertyIndexer] = {};
            results[propertyIndexer][_.id] = { message: validated ? "" : _.validationMessage, success: validated };
        });
    }
    else {
        if (rule.fromType != undefined) {
            let childSchema = yield FormSchemaUtil_1.FormSchemaUtil.GetSchemaFromItem(rule.fromType);
            if (item[property] != null) {
                results = yield exports.formValidator.validate(item[property], undefined, childSchema, onEvent, results, property); // TODO: maybe set all events here ('undefined'), to prevent validation messages to dissapear when other properties change (ewi)
            }
        }
        else {
            let validated = (onEvent != undefined && !((rule.validateOn & onEvent) === onEvent)) || (!rule.disabled && rule.validationRule(item));
            if (results[propertyIndexer] == null)
                results[propertyIndexer] = {};
            results[propertyIndexer][rule.id] = { message: validated ? "" : rule.validationMessage, success: validated };
        }
    }
});
exports.formValidator = {
    // TODO: handle nested validation on complex objects, more smooth (ewi)
    // TODO: handle validation differently than using 'onEvent'?! (ewi)
    validate: (item, prop, schema, onEvent, existingResults, validationResultPrefix) => __awaiter(void 0, void 0, void 0, function* () {
        var _a;
        let validation = (_a = schema === null || schema === void 0 ? void 0 : schema.options) === null || _a === void 0 ? void 0 : _a.validation;
        if (validation == null)
            return existingResults;
        if (validation.ignoreValidation)
            return existingResults;
        let rules = validation.validationRules;
        if (rules == null)
            return existingResults;
        let results = existingResults !== null && existingResults !== void 0 ? existingResults : {};
        // specific prop needs validation
        if (prop !== undefined) {
            let rule = rules[prop];
            yield validateRule(item, prop, rule, onEvent, results, validationResultPrefix);
        }
        else {
            for (const key of Object.keys(rules)) {
                let rule = rules[key];
                yield validateRule(item, key, rule, onEvent, results, validationResultPrefix);
            }
        }
        return results;
    })
};
//# sourceMappingURL=FormValidator.js.map